require 'yaml'

require 'punt/mode/mode_scp'
require 'punt/repo/repo_git'

require 'punt/helper/asset_helper'
require 'punt/helper/puntfile_helper'
require 'punt/helper/mode_helper'
require 'punt/helper/repo_helper'

require 'punt/deploy'
require 'punt/remote'

require 'punt/cmd/cmd'
require 'punt/cmd/cmd_init'
require 'punt/cmd/cmd_deploy'
require 'punt/cmd/cmd_remote'

class Punt
    CMDS = [CmdInit.new,
            CmdDeploy.new,
            CmdRemote.new,
    ].sort { |first, second| first.name <=> second.name }

    def self.cmd(argv)
        first = argv.shift
        matching_cmd = find_cmd(first)

        usage and return unless matching_cmd

        matching_cmd.run(argv)
    end

    private
    def self.find_cmd(arg)
        return CMDS.reject {|cmd| cmd.name != arg }.first
    end

    def self.usage()
        puts "Usage: punt [cmd] [cmd-arguments]"
        puts ""
        puts "punt recognizes the following commands:"
        puts ""
        CMDS.each do |cmdClazz|
            puts "\t#{cmdClazz.name} - #{cmdClazz.summary}"
        end
    end
end
