module PuntfileHelper

    SUPPORTED_PUNTFILES = ["puntfile.yml", "puntfile.yaml", "Puntfile.yml", "Puntfile.yaml", "puntfile", "Puntfile"]

    # Returns the puntfile hash for the current program. This will scan the various supported Puntfile
    # file formats and attempt to load the first one it finds.
    def puntfile
        load_puntfile

        return @@puntfile
    end

    # Pulls the specific environment hash out of the Puntfile. If no environment is
    # given, this automatically returns the default environment.
    def punt_environment(env: nil)
        env = puntfile.first.first unless env
        environment = puntfile[env] if env

        if (!environment)
            raise "No environment #{env} was found in the puntfile"
        end

        return environment
    end

    private

    def load_puntfile
        return if defined?(@@puntfile)

        SUPPORTED_PUNTFILES.each do |puntfile|
            if File.exists?(puntfile)
                @@puntfile = YAML.load_file(puntfile)
                return
            end
        end

        raise "We couldn't find puntfile.yml in the current directory"
    end


end
